#include <iomanip>
#include <iostream>
#include <cmath>
#include <vector>
#define n 7

using namespace std;

void gaussianElimination(int m1, double** matrix, double* vec, vector<double>& result) {
    for (int i = 0; i < m1 - 1; i++) {
        for (int j = i + 1; j < m1; j++) {
            double factor = matrix[j][i] / matrix[i][i];
            for (int k = i; k < m1; k++) {
                matrix[j][k] -= factor * matrix[i][k];
            }
            vec[j] -= factor * vec[i];
        }
    }
    for (int i = m1 - 1; i >= 0; i--) {
        double sum = 0;
        for (int j = i + 1; j < m1; j++) {
            sum += matrix[i][j] * result[j];
        }
        result[i] = (vec[i] - sum) / matrix[i][i];
    }
}

void regression(double* x, const double* y, int m) {
    double** matrix = new double*[m + 1];
    for (int i = 0; i <= m; ++i) {
        matrix[i] = new double[m + 1]();
    }
    double* vec = new double[m + 1]();
    for (int i = 0; i <= m; i++) {
        for (int j = 0; j <= m; j++) {
            matrix[i][j] = 0;
            for (int k = 0; k < n; k++) {
                matrix[i][j] += pow(x[k], i + j);
            }
        }
        vec[i] = 0;
        for (int k = 0; k < n; k++) {
            vec[i] += y[k] * pow(x[k], i);
        }
    }
#ifdef OFFICE_OUTPUT
    cout<<"newline"<<endl;
#endif
    vector<double> result(m + 1);
    gaussianElimination(m + 1, matrix, vec, result);
    
    cout << "Regression (m = " << m << "): "<<char(101+m)<<"(x)=";
    for (int j = 0; j <= m; j++) {
        if(j)cout<<showpos;
        cout << result[j]<<setprecision(60);
        if(j)cout<<'x'<<noshowpos;
        if(j>1)cout<<'^'<<j;
    }
    cout << noshowpos << endl;
    
    for (int i = 0; i <= m; ++i) {
        delete[] matrix[i];
    }
    delete[] matrix;
    delete[] vec;
}

#define display(x) cout << #x << ":"; for (char a = 0; a < n; a++) cout << ' ' << x[a]; cout << endl;

int main() {
    double y[] = {0.1305, 0.018821, 0.267441, 4.128129, 64.711993,1025.884364};
    double x[] = {0,10, 12, 14, 16, 18, 20};
    cout << setprecision(2) << fixed;
    regression(x, y, 1);
    regression(x, y, 2);
    regression(x, y, 3);
    
    return 0;
}
